
/*
 * A simple model for the maze
 *
 * A maze is a connected collections of cells
 * where a cell has exits in four directions
 * each direction holding either none() or
 * some(c) for some cell c
 *
 */


class BasicModel (view:View, controller:Controller) extends Model  {

  /* The current position of the player in the maze */
  private var cell:Cell = initializeMaze()

  /* The current direction faced by the player in the maze */
  private var direction:Direction = Direction.NORTH

  // constructor 
  view.attachToModel(this)
  controller.attachToModel(this)
  // initial display of information
  updateView()



  private class BasicVisible (d:Direction,
			      l:Boolean,r:Boolean,f:Boolean,
			      fl:Boolean,fr:Boolean,ff:Boolean,
			      ffl:Boolean,ffr:Boolean,fff:Boolean,
			      fw:Boolean,ffw:Boolean) extends Visible {

    def direction ():Direction = d
    def left ():Boolean = l
    def right ():Boolean = r
    def front ():Boolean = f
    def frontleft ():Boolean = fl
    def frontright ():Boolean = fr
    def frontfront ():Boolean = ff
    def frontfrontleft ():Boolean = ffl
    def frontfrontright ():Boolean = ffr
    def frontfrontfront ():Boolean = fff
    def frontwin ():Boolean = fw
    def frontfrontwin ():Boolean = ffw
  }

  def isOpening (oc:Option[Cell]):Boolean = !(oc.isNone())

  def updateView ():Unit = {
    val l = isOpening(cell.exit(direction.left()))
    val r = isOpening(cell.exit(direction.right()))
    val front = cell.exit(direction);
    if (!isOpening(front)) {
      view.update(new BasicVisible(direction,l,r,false,false,false,false,
				   false,false,false,false,false))
      return
    }
    val fl = isOpening(front.valOf().exit(direction.left()))
    val fr = isOpening(front.valOf().exit(direction.right()))
    val ffront = front.valOf().exit(direction)
    val fwin = front.valOf().isWin()
    if (!isOpening(ffront)) {
      view.update(new BasicVisible(direction,l,r,true,fl,fr,false,
				   false,false,false,fwin,false))
      return
    }
    val ffl = isOpening(ffront.valOf().exit(direction.left()))
    val ffr = isOpening(ffront.valOf().exit(direction.right()))
    val fffront = ffront.valOf().exit(direction)
    val ffwin = ffront.valOf().isWin()
    view.update(new BasicVisible(direction,l,r,true,fl,fr,true,
				 ffl,ffr,isOpening(fffront),
				 fwin,ffwin))
    return
  }


  private def initializeMaze ():Cell = {

    val cells:Array[Cell] = new Array[Cell](8)

    // create cells
    for (i <- 0 to 7)
      cells(i) = new Cell
    val wincell = new WinCell

    // connect cells
    cells(0).connect(Direction.SOUTH,cells(1));
    cells(1).connect(Direction.SOUTH,cells(2));
    cells(2).connect(Direction.EAST,cells(3));
    cells(3).connect(Direction.EAST,cells(4));
    cells(4).connect(Direction.NORTH,cells(5));
    cells(5).connect(Direction.NORTH,cells(6));
    cells(6).connect(Direction.WEST,cells(7));
    cells(7).connect(Direction.WEST,cells(0));
    
    cells(4).connect(Direction.SOUTH,wincell);
    
    // choose starting cell 
    cells(0);
  }


  def performLeft ():Boolean =  {
    direction=direction.left()
    updateView()
    true
  }

  def performRight ():Boolean = {
    direction=direction.right()
    updateView()
    true
  }

  def performForward ():Boolean = {
    val res = cell.exit(direction)
    if (res.isNone())
      false
    else {
      cell=res.valOf()
      updateView()
      if (cell.isWin()) 
	view.win()
      true
    }
  }

  def performBack ():Boolean = {
    val res = cell.exit(direction.opposite())
    if (res.isNone())
      false 
    else {
      cell=res.valOf()
      updateView()
      if (cell.isWin())
	view.win()
      true
    }
  }
}
