
/* 
 * A cell in the maze representation
 *
 * A cell has exits in four directions
 * each direction holding either none() if there's
 * a wall in that direction, or some(c) if the
 * direction leads to a new cell c
 *
 * These exits are held in an ArrayList from the
 * Java Collections library
 *
 */

class Cell {

  import scala.collection.mutable.ArrayBuffer

  val exits:ArrayBuffer[Option[Cell]] = new ArrayBuffer(4)

  for (i <- 0 to 3)
    exits.append(Option.none())

  
  def exit (d:Direction):Option[Cell] = exits(d.ordinal())

  /* attempt to connect the current cell to another cell
  * stop if the current cell already has an exit in that
  * direction 
  */
  def connect (d:Direction, c:Cell):Unit = 
    if (exits(d.ordinal()).isNone()) {
      exits(d.ordinal()) = Option.some(c)
      c.connect(d.opposite(),this);
    }

  def isWin ():Boolean  = false
}



/*
 * A special kind of cell that identifies a "winning" cell
 *
 * When you hit a winning cell, you win the game
 *
 */

class WinCell extends Cell {

  override def isWin ():Boolean = true
}
