
/*
 * Some rather clunky code to get a simple window going that can 
 * be used to draw lines, and to get key presses
 *
 * The GUI view uses it to draw a pictorial representation of the maze
 * and the GUI controller uses it to get key press inputs to control
 * the player
 *
 * This is probably not the best way to use the GUI features of Java...
 *
 */

import java.awt.Canvas
import java.awt.Graphics
import java.awt._
import java.awt.event._
import java.util._
import java.lang.Character


object GUI {

  private class Quadtuple (ax1:Int, ay1:Int, ax2:Int, ay2:Int) {
    val x1 = ax1
    val y1 = ay1
    val x2 = ax2
    val y2 = ay2
  }	
    
  private class CanvasRegion extends Canvas { 
    private var lines:List[Quadtuple] = List.empty()
    def addLine (l:Quadtuple):Unit = 
      lines = List.cons(l,lines)

    def clearLines ():Unit = 
      lines = List.empty()

    val sides = 500

    def CanvasRegion():Unit = {  
      setSize(sides, sides)
    } 

    override def paint(g:Graphics):Unit = { 
      for (q <- lines) 
	g.drawLine(q.x1,q.y1,q.x2,q.y2)
    } 
  } 
    
  private class ClickyMouseAdapter (parentFrame:CanvasFrame) 
	                                     extends MouseAdapter {
	
    override def mouseClicked (e:MouseEvent):Unit = {
      parentFrame.clickEvent()
    }
  }

  private class PressKeyAdapter (parentFrame:CanvasFrame) extends KeyAdapter {
    override def keyTyped (e:KeyEvent):Unit = {
      parentFrame.keyEvent(e.getKeyChar())
    }
  }
    
  private class CanvasFrame extends Frame("Maze") { 
    private val region:CanvasRegion = new CanvasRegion ()
    private var click:Boolean = false

    setSize(region.sides+30, region.sides+30); 
    add(region); 
    addWindowListener(new WindowAdapter() { 
      override def windowClosing(e:WindowEvent):Unit = { 
	setVisible(false)
	dispose()
	System.exit(0)
      } 
    })
    region.addMouseListener(new ClickyMouseAdapter(this))
    region.addKeyListener(new PressKeyAdapter(this))

    def clickEvent ():Unit = {
      this.click = true
    }

    def keyEvent(c:Character) = {
      if (keyObserver.isSome())
	keyObserver.valOf().notify(c)
    }
	    
    def drawLine (x1:Int, y1:Int, x2:Int, y2:Int):Unit = {
      this.region.addLine(new Quadtuple(x1,region.sides-y1,
					x2,region.sides-y2))
    }
    
    def clear ():Unit = {
      this.region.clearLines
      this.region.repaint()
    }
	
    def update ():Unit = {
      this.region.repaint()
    }
	
    def waitForClick ():Unit = {
      this.click = false
      while (!(this.click)) {
	try {
	  Thread.sleep(500);
	} catch {
	  case e:InterruptedException => ()
	}
      }
    }
  }
    
  private var canvasFrame:Option[CanvasFrame] = Option.none()

  private var keyObserver:Option[Observer[Character]] = Option.none()

  def registerKeyObserver (ko:Observer[Character]):Unit = {
    if (keyObserver.isNone())
      keyObserver = Option.some(ko)
  }

  def initialize ():Unit = {
    if (canvasFrame.isNone()) {
      canvasFrame = Option.some(new CanvasFrame())
      canvasFrame.valOf()setVisible(true)
    }
  }
    
  def drawLine (x1:Int, y1:Int, x2:Int, y2:Int) = {
    if (canvasFrame.isNone()) 
      throw new Error ("GUI not initialized")
    canvasFrame.valOf().drawLine(x1,y1,x2,y2)
  }

  def clear ():Unit = {
    if (canvasFrame.isNone())
      throw new Error ("GUI not initialized")
    canvasFrame.valOf().clear()
  }

  def redraw ():Unit = {
    if (canvasFrame.isNone())
      throw new Error ("GUI not initialized")
    canvasFrame.valOf().update()
  }

}
