
/*
 * A controller for the maze game that uses key presses in the GUI window 
 * to determine which actions to send to the model
 *
 * This is clunky because getting key presses from the window is clunky,
 * at least given the way I've set things up
 *
 * Roughly, this defines an observer for key presses that gets
 * registered in the GUI so that we get notified whenever there is 
 * a key pressed
 *
 * CONTROLS:
 *  i  - forward
 *  k  - back
 *  j  - left
 *  l  - right
 *  q  - quit
 *
 */

import java.io._
import java.lang.Character


class GUIController extends Controller {

  GUI.initialize()

  private var maze:Option[Model] = Option.none()

  def attachToModel (m:Model):Unit =
    maze = Option.some(m)

  private class KeyObserver (maze:Model) extends Observer[Character] {

    def notify (c:Character):Unit = 
      if (c.charValue()=='i')
	maze.performForward()
      else if (c.charValue()=='k')
	maze.performBack()
      else if (c.charValue()=='j')
	maze.performLeft()
      else if (c.charValue()=='l')
	maze.performRight()
      else if (c.charValue()=='q')
	System.exit(0)
  }
  
  def run ():Unit = { 
    if (maze.isNone())
      throw new Error("GUIController not initialized with model")
    val ko:KeyObserver = new KeyObserver(maze.valOf())
    GUI.registerKeyObserver(ko)
  }
}
	
